#!/bin/bash

# Copyright (c) 2012 Cloudera, Inc. All rights reserved.

set -e
set -x

# Time marker for both stderr and stdout
date; date 1>&2

cloudera_config=`dirname $0`
cloudera_config=`cd "$cloudera_config/../common"; pwd`
. ${cloudera_config}/cloudera-config.sh

# Load the parcel environment
source_parcel_environment

export IMPALA_HOME=$CDH_IMPALA_HOME

# set impala configuration directory
export IMPALA_CONF_DIR=${CONF_DIR}/impala-conf
export HADOOP_CONF_DIR=${CONF_DIR}/hadoop-conf
export HIVE_CONF_DIR=${CONF_DIR}/hive-conf
export HBASE_CONF_DIR=${CONF_DIR}/hbase-conf

# Add JDBC jars to the path. Oracle and Mysql installed by end user, can't ship due to licensing.
JDBC_JARS="$CLOUDERA_MYSQL_CONNECTOR_JAR:$CLOUDERA_POSTGRESQL_JDBC_JAR:$CLOUDERA_ORACLE_CONNECTOR_JAR"
if [[ -z "$AUX_CLASSPATH" ]]; then
  export AUX_CLASSPATH="$JDBC_JARS"
else
  export AUX_CLASSPATH="$AUX_CLASSPATH:$JDBC_JARS"
fi
# Due to IMP-898 Impala isn't correctly picking up AUX_CLASSPATH in Impala 1.0, but it does pick up CLASSPATH.
# Just emit both.
if [[ -z "$CLASSPATH" ]]; then
  export CLASSPATH="$JDBC_JARS"
else
  export CLASSPATH="$CLASSPATH:$JDBC_JARS"
fi

FLAG_FILE=$IMPALA_CONF_DIR/$2
USE_DEBUG_BUILD=$3

# Update path to configuration directory in the flagfile
perl -pi -e "s#{{CMF_CONF_DIR}}#$CONF_DIR#g" ${FLAG_FILE}

# Set Impala bin location if debug build need to run. The /etc/default/impala
# will use this location and when we exec impala bash file below, it will
# use the impala binary from debug build location
if $USE_DEBUG_BUILD ; then
  export IMPALA_BIN=${IMPALA_HOME}/sbin-debug
else
  export IMPALA_BIN=${IMPALA_HOME}/sbin-retail
fi

if [ "impalad" = "$1" ]; then
  exec "$IMPALA_HOME/../../bin/impalad" --flagfile=${FLAG_FILE}
elif [ "statestore" = "$1" ]; then
  exec "$IMPALA_HOME/../../bin/statestored" --flagfile=${FLAG_FILE}
fi
